@@ -44,7 +44,7 @@ from pysollib.stack import \
         UD_SS_RowStack, \
         WasteStack, \
         WasteTalonStack
-from pysollib.util import ACE, ANY_RANK, KING, NO_RANK, QUEEN, RANKS,\
+from pysollib.util import ACE, ANY_RANK, KING, NO_RANK, QUEEN, RANKS, \
         UNLIMITED_ACCEPTS, UNLIMITED_MOVES
 
 
@@ -376,6 +376,7 @@ class CastlesEnd(Bastion):
 # ************************************************************************
 # * Chessboard
 # * Lasker
+# * Morphy
 # ************************************************************************
 
 class Chessboard_Foundation(SS_FoundationStack):
@@ -431,10 +432,24 @@ class Lasker(Chessboard):
                                   max_accept=UNLIMITED_ACCEPTS)
 
 
+class Morphy_RowStack(UD_AC_RowStack):
+    def canDropCards(self, stacks):
+        if self.game.demo:
+            return UD_AC_RowStack.canDropCards(self, stacks)
+        for s in self.game.s.foundations:
+            if s.cards:
+                return UD_AC_RowStack.canDropCards(self, stacks)
+        return (None, 0)
+
+
+class Morphy(Chessboard):
+    RowStack_Class = StackWrapper(Morphy_RowStack, mod=13)
+
+
 # ************************************************************************
 # * Siegecraft
 # * Stronghold
-# * Fastness
+# * Private Lane
 # ************************************************************************
 
 class Siegecraft(BeleagueredCastle):
@@ -453,7 +468,7 @@ class Stronghold(StreetsAndAlleys):
         StreetsAndAlleys.createGame(self, reserves=1)
 
 
-class Fastness(StreetsAndAlleys):
+class PrivateLane(StreetsAndAlleys):
     Hint_Class = FreeCellType_Hint
     Solver_Class = FreeCellSolverWrapper(sbb='rank')
 
@@ -885,7 +900,7 @@ registerGame(GameInfo(34, BeleagueredCastle, "Beleaguered Castle",
                       altnames=("Laying Siege", "Sham Battle",)))
 registerGame(GameInfo(145, Citadel, "Citadel",
                       GI.GT_BELEAGUERED_CASTLE | GI.GT_OPEN, 1, 0,
-                      GI.SL_MOSTLY_SKILL))
+                      GI.SL_MOSTLY_SKILL, altnames=('Under Siege',)))
 registerGame(GameInfo(147, Fortress, "Fortress",
                       GI.GT_BELEAGUERED_CASTLE | GI.GT_OPEN, 1, 0,
                       GI.SL_SKILL))
@@ -895,9 +910,9 @@ registerGame(GameInfo(148, Chessboard, "Chessboard",
 registerGame(GameInfo(300, Stronghold, "Stronghold",
                       GI.GT_BELEAGUERED_CASTLE | GI.GT_OPEN, 1, 0,
                       GI.SL_MOSTLY_SKILL))
-registerGame(GameInfo(301, Fastness, "Fastness",
+registerGame(GameInfo(301, PrivateLane, "Private Lane",
                       GI.GT_BELEAGUERED_CASTLE | GI.GT_OPEN, 1, 0,
-                      GI.SL_MOSTLY_SKILL, altnames=('Private Lane',)))
+                      GI.SL_MOSTLY_SKILL, altnames=('Fastness',)))
 registerGame(GameInfo(306, Zerline, "Zerline",
                       GI.GT_BELEAGUERED_CASTLE, 2, 0, GI.SL_MOSTLY_SKILL))
 registerGame(GameInfo(324, Bastion, "Bastion",
@@ -943,3 +958,6 @@ registerGame(GameInfo(831, Siegecraft, "Siegecraft",
 registerGame(GameInfo(881, Lasker, "Lasker",
                       GI.GT_BELEAGUERED_CASTLE | GI.GT_OPEN, 1, 0,
                       GI.SL_SKILL))
+registerGame(GameInfo(951, Morphy, "Morphy",
+                      GI.GT_BELEAGUERED_CASTLE | GI.GT_OPEN, 1, 0,
+                      GI.SL_SKILL))
