@@ -448,8 +448,8 @@ class DoubleLarasGame(RelaxedLarasGame):
 
 registerGame(GameInfo(37, LarasGame, "Lara's Game",
                       GI.GT_2DECK_TYPE, 2, 0, GI.SL_BALANCED,
-                      altnames=("Thirteen Packs",)))
+                      altnames=("Thirteen Packs", "Leoni's Own", "Weavers")))
 registerGame(GameInfo(13006, RelaxedLarasGame, "Lara's Game Relaxed",
                       GI.GT_2DECK_TYPE, 2, 1, GI.SL_BALANCED))
 registerGame(GameInfo(13007, DoubleLarasGame, "Lara's Game Doubled",
-                      GI.GT_2DECK_TYPE, 4, 2, GI.SL_BALANCED))
+                      GI.GT_4DECK_TYPE, 4, 2, GI.SL_BALANCED))
