@@ -32,6 +32,7 @@ from pysollib.mygettext import _
 from pysollib.pysoltk import MfxCanvasText
 from pysollib.stack import \
         AC_RowStack, \
+        AbstractFoundationStack, \
         BasicRowStack, \
         DealRowTalonStack, \
         InitialDealTalonStack, \
@@ -1196,6 +1197,210 @@ class Ladybug(Game):
         return [self.used]
 
 
+# ************************************************************************
+# * The Bogey
+# ************************************************************************
+
+class TheBogey_Foundation(SS_FoundationStack):
+    def acceptsCards(self, from_stack, cards):
+        if not self.cards:
+            return 1
+        return (self.cards[-1].rank > cards[0].rank
+                and self.cards[-1].suit == cards[0].suit)
+
+    def canMoveCards(self, cards):
+        return False
+
+
+class TheBogey_BogeyDraw(OpenStack):
+    def moveMove(self, ncards, to_stack, frames=-1, shadow=-1):
+        OpenStack.moveMove(self, ncards, to_stack, frames=frames,
+                           shadow=shadow)
+        rows = [r for r in self.game.s.rows if not len(r.cards) > 0]
+        self.game.startDealSample()
+        self.game.s.talon.dealRowAvail(rows=rows)
+        rows2 = [r for r in self.game.s.rows if not len(r.cards) > 0]
+        if len(rows2) > 0:
+            self.game.s.talon.redeal(sound=False)
+            self.game.s.talon.dealRowAvail(rows=rows2)
+        self.game.stopSamples()
+
+
+class TheBogey_RowStack(OpenStack):
+
+    def canMoveCards(self, cards):
+        if len(self.game.s.reserves[0].cards) > 0:
+            return False
+        return OpenStack.canMoveCards(self, cards)
+
+
+class TheBogey_Talon(TalonStack):
+    def canDealCards(self):
+        return (len(self.game.s.reserves[0].cards) < 1 and
+                (len(self.cards) > 0 or
+                 len(self.game.s.reserves[0].cards) > 0))
+
+    def dealCards(self, sound=False):
+        if len(self.cards) < 1:
+            self.redeal(sound=sound)
+        if sound:
+            self.game.playSample("dealwaste")
+        self.flipMove()
+        self.moveMove(1, self.game.s.reserves[0])
+
+    def redeal(self, sound=False):
+        if len(self.game.s.reserves[1].cards) > 0:
+            assert len(self.cards) == 0
+            if sound:
+                self.game.playSample("turnwaste", priority=20)
+            self.game.turnStackMove(self.game.s.reserves[1], self)
+            self.game.shuffleStackMove(self)
+            self.game.nextRoundMove(self)
+
+
+class TheBogey_Discard(OpenStack):
+    getBottomImage = Stack._getReserveBottomImage
+
+    def acceptsCards(self, from_stack, cards):
+        return len(self.game.s.reserves[0].cards) < 1
+
+    def canMoveCards(self, cards):
+        return False
+
+
+class TheBogey(Game):
+
+    def createGame(self):
+        # create layout
+        l, s = Layout(self), self.s
+
+        # set window
+        self.setSize(l.XM + 12 * l.XS, l.YM + 2 * l.YS + 13 * l.YOFFSET)
+
+        # create stacks
+        x, y = l.XM, l.YM
+        for i in range(12):
+            s.foundations.append(TheBogey_Foundation(x, y, self, dir=-1,
+                                                     suit=ANY_SUIT))
+            s.foundations[i].CARD_YOFFSET = l.YOFFSET
+            x += l.XS
+
+        x, y = l.XM, self.height - l.YS
+
+        s.talon = TheBogey_Talon(x, y, self, max_rounds=-1)
+        l.createText(s.talon, 'n')
+
+        x += l.XS
+        s.reserves.append(TheBogey_BogeyDraw(x, y, self))
+
+        x += 3 * l.XS
+
+        for i in range(5):
+            s.rows.append(TheBogey_RowStack(x, y, self, max_cards=1,
+                                            max_accept=0))
+            x += l.XS
+
+        x += 2 * l.XS
+        s.reserves.append(TheBogey_Discard(x, y, self))
+        l.createText(s.reserves[1], 'n')
+
+        self.setRegion(s.reserves, (-999, y - l.CH // 2, 999999, 999999))
+
+        # define stack-groups
+        l.defaultStackGroups()
+
+    def startGame(self):
+        self.startDealSample()
+        self.s.talon.dealRow(rows=self.s.rows)
+
+
+# ************************************************************************
+# * Ninety-One
+# ************************************************************************
+
+class NinetyOne_RowStack(OpenStack):
+
+    def moveMove(self, ncards, to_stack, frames=-1, shadow=-1):
+        OpenStack.moveMove(self, ncards, to_stack, frames=frames,
+                           shadow=shadow)
+        if to_stack in self.game.s.rows:
+            self.game.checkTotal()
+
+
+class NinetyOne(Game):
+    Hint_Class = None
+
+    def createGame(self):
+        # create layout
+        l, s = Layout(self), self.s
+
+        self.setSize(l.XM + 7 * l.XS, l.YM + 2 * l.YS)
+
+        x, y = l.XM, l.YM
+        # create stacks
+        for j in range(7):
+            s.rows.append(NinetyOne_RowStack(x, y, self, max_move=1,
+                                             max_accept=1))
+            x += l.XS
+        x, y = l.XM, l.YM + l.YS
+        for j in range(6):
+            s.rows.append(NinetyOne_RowStack(x, y, self, max_move=1,
+                                             max_accept=1))
+            x += l.XS
+
+        # create text
+        if self.preview <= 1:
+            y += l.YS // 2
+            self.texts.score = MfxCanvasText(
+                self.canvas, x, y, anchor="sw",
+                font=self.app.getFont("canvas_large"))
+
+        x, y = self.getInvisibleCoords()
+        s.talon = InitialDealTalonStack(x, y, self)
+        s.foundations.append(AbstractFoundationStack(x, y, self,
+                                                     max_move=0,
+                                                     max_accept=0,
+                                                     suit=ANY_SUIT))
+
+        # define stack-groups
+        l.defaultStackGroups()
+
+    def startGame(self):
+        for i in range(3):
+            self.s.talon.dealRow(frames=0)
+        self._startAndDealRow()
+
+    def updateText(self):
+        if self.preview > 1 or not self.texts.score:
+            return
+        self.texts.score.config(text=self.getTotal())
+
+    def getTotal(self):
+        total = 0
+        for r in self.s.rows:
+            if len(r.cards) == 0:
+                continue
+            total += r.cards[-1].rank + 1
+        return total
+
+    def checkTotal(self):
+        self.updateText()
+        if self.getTotal() == 91:
+            for r in self.s.rows:
+                if len(r.cards) == 0:
+                    return
+            self.startDealSample()
+            old_state = self.enterState(self.S_FILL)
+            for r in self.s.rows:
+                r.moveMove(1, self.s.foundations[0])
+            self.leaveState(old_state)
+            self.stopSamples()
+            self.checkTotal()
+
+    def getAutoStacks(self, event=None):
+        return ((), (), ())
+
+
 # register the game
 registerGame(GameInfo(257, Numerica, "Numerica",
                       GI.GT_NUMERICA | GI.GT_CONTRIB, 1, 0, GI.SL_BALANCED,
@@ -1215,7 +1420,8 @@ registerGame(GameInfo(379, Chamberlain, "Chamberlain",
                       GI.GT_NUMERICA | GI.GT_OPEN | GI.GT_ORIGINAL, 1, 0,
                       GI.SL_MOSTLY_SKILL))
 registerGame(GameInfo(402, Toad, "Toad",
-                      GI.GT_NUMERICA, 2, 0, GI.SL_BALANCED))
+                      GI.GT_NUMERICA, 2, 0, GI.SL_BALANCED,
+                      altnames=("Toad in the Hole")))
 registerGame(GameInfo(430, PussInTheCorner, "Puss in the Corner",
                       GI.GT_NUMERICA, 1, 1, GI.SL_BALANCED))
 registerGame(GameInfo(435, Shifting, "Shifting",
@@ -1250,3 +1456,7 @@ registerGame(GameInfo(836, Ladybug, "Ladybug",
                       GI.GT_1DECK_TYPE, 1, -2, GI.SL_BALANCED))
 registerGame(GameInfo(899, Housefly, "Housefly",
                       GI.GT_NUMERICA, 1, 0, GI.SL_BALANCED))
+registerGame(GameInfo(931, TheBogey, "The Bogey",
+                      GI.GT_1DECK_TYPE, 1, -1, GI.SL_BALANCED))
+registerGame(GameInfo(958, NinetyOne, "Ninety-One",
+                      GI.GT_1DECK_TYPE, 1, 0, GI.SL_MOSTLY_SKILL))
