@@ -21,6 +21,8 @@
 #
 # ---------------------------------------------------------------------------#
 
+from collections import UserList
+
 from kivy.clock import Clock
 
 from pysollib.gamedb import GI
@@ -28,12 +30,11 @@ from pysollib.kivy.LApp import LScrollView
 from pysollib.kivy.LApp import LTopLevel
 from pysollib.kivy.LApp import LTreeNode
 from pysollib.kivy.LApp import LTreeRoot
+from pysollib.kivy.LApp import get_menu_size_hint
 from pysollib.kivy.selecttree import SelectDialogTreeData
 from pysollib.kivy.selecttree import SelectDialogTreeLeaf, SelectDialogTreeNode
 from pysollib.mygettext import _
 
-from six.moves import UserList
-
 
 # ************************************************************************
 # * Nodes
@@ -93,9 +94,12 @@ class SelectGameData(SelectDialogTreeData):
                      ):
             gg = []
             for name, select_func in data:
-                if name is None or not filter(select_func, self.all_games_gi):
+                filtered = filter(select_func, self.all_games_gi)
+                if name is None or not filtered or len(list(filtered)) == 0:
                     continue
-                gg.append(SelectGameNode(None, _(name), select_func))
+                node = SelectGameNode(None, _(name), select_func)
+                if node:
+                    gg.append(node)
             g.append(gg)
 
         def select_mahjongg_game(gi): return gi.si.game_type == GI.GT_MAHJONGG
@@ -105,7 +109,7 @@ class SelectGameData(SelectDialogTreeData):
                                 select_mahjongg_game)
         g.append(gg)
         if g[0]:
-            s_by_type = SelectGameNode(None, _("French games"),
+            s_by_type = SelectGameNode(None, _("French Games"),
                                        tuple(g[0]), expanded=1)
         if g[1]:
             s_oriental = SelectGameNode(None, _("Oriental Games"),
@@ -165,7 +169,7 @@ class SelectGameData(SelectDialogTreeData):
             if name is None or not list(filter(
                     select_func, self.all_games_gi)):
                 continue
-            name = _("New games in v. ") + name
+            name = _("New games in v. %(version)s") % {'version': name}
             gg.append(SelectGameNode(None, name, select_func))
         if 1 and gg:
             s_by_pysol_version = SelectGameNode(None, _("by PySol version"),
@@ -251,11 +255,11 @@ class SelectGameData(SelectDialogTreeData):
                                lambda gi: gi.si.redeals == 3),
                     SelectGameNode(None, _("Unlimited redeals"),
                                lambda gi: gi.si.redeals == -1),
-                    SelectGameNode(None, "Variable redeals",
+                    SelectGameNode(None, _("Variable redeals"),
                                lambda gi: gi.si.redeals == -2),
                     SelectGameNode(None, _("Other number of redeals"),
                                lambda gi: gi.si.redeals not in
-                               (-1, 0, 1, 2, 3)),
+                               (-2, -1, 0, 1, 2, 3)),
                 )),
                 s_by_compatibility,
             )),
@@ -266,8 +270,13 @@ class SelectGameData(SelectDialogTreeData):
                                lambda gi: gi.si.game_flags & GI.GT_CHILDREN),
                 SelectGameNode(None, _("Games with Scoring"),
                                lambda gi: gi.si.game_flags & GI.GT_SCORE),
+                SelectGameNode(None, _("Games with Stripped Decks"),
+                               lambda gi: gi.si.game_flags & GI.GT_STRIPPED),
                 SelectGameNode(None, _("Games with Separate Decks"),
                            lambda gi: gi.si.game_flags & GI.GT_SEPARATE_DECKS),
+                SelectGameNode(None, _("Games with Jokers"),
+                               lambda gi: gi.category == GI.GC_FRENCH and
+                               gi.subcategory == GI.GS_JOKER_DECK),
                 SelectGameNode(None, _("Open Games (all cards visible)"),
                                lambda gi: gi.si.game_flags & GI.GT_OPEN),
                 SelectGameNode(None, _("Relaxed Variants"),
@@ -380,7 +389,6 @@ class SelectGameDialog(object):
     def onClick(self, event):
         print('LTopLevel: onClick')
         SelectGameDialog.SingleInstance.parent.popWork('SelectGame')
-        SelectGameDialog.SingleInstance.running = False
 
     def selectCmd(self, gameid):
         self.app.menubar._mSelectGame(gameid)
@@ -392,31 +400,31 @@ class SelectGameDialog(object):
         self.app = app
         self.gameid = gameid
         self.random = None
-        self.running = False
         self.window = None
 
         # bestehenden Dialog rezyklieren.
 
         si = SelectGameDialog.SingleInstance
-        # if (si and si.running): return
-        if (si and si.running):
-            si.parent.popWork('SelectGame')
-            si.running = False
+        if si and si.parent.workStack.peek('SelectGame') is not None:
+            parent.popWork('SelectGame')
             return
         if (si):
             si.parent.pushWork('SelectGame', si.window)
-            si.running = True
             return
 
         # neuen Dialog aufbauen.
 
-        window = LTopLevel(parent, title)
-        window.titleline.bind(on_press=self.onClick)
-        self.parent.pushWork('SelectGame', window)
-        self.window = window
-        self.running = True
+        self.window = window = LTopLevel(parent, title)
+        self.window.titleline.bind(on_press=self.onClick)
+
+        self.parent.pushWork('SelectGame', self.window)
         SelectGameDialog.SingleInstance = self
 
+        def updrule(obj, val):
+            self.window.size_hint = get_menu_size_hint()
+        updrule(0, 0)
+        self.parent.bind(size=updrule)
+
         # Asynchron laden.
 
         def loaderCB(treeview, node):
@@ -444,10 +452,10 @@ class SelectGameDialog(object):
                 nodes = n.getContents()
                 if type(nodes) is list:
                     # Blaetter
-                    for l in nodes:
-                        # print ('**game=%s' % l.text)
+                    for node in nodes:
+                        # print ('**game=%s' % node.text)
                         yield LGameNode(
-                            l, v, text=l.text,
+                            node, v, text=node.text,
                             is_leaf=True,
                             command=self.selectCmd)
 
@@ -475,7 +483,7 @@ class SelectGameDialog(object):
             tree,
             self.app.canvas,
             root_options=dict(text='Tree One'))
-        tv.size_hint = 1, None
+        tv.size_hint = (1, None)
         tv.hide_root = True
         tv.load_func = loaderCB
         tv.bind(minimum_height=tv.setter('height'))
