@@ -25,23 +25,41 @@ import os
 import sys
 import traceback
 
+import configobj
 
 import pysollib.settings
-from pysollib.configobj import configobj, validate
-from pysollib.mfxutil import print_err
+from pysollib.mfxutil import USE_PIL, \
+    get_default_resampling, print_err
 from pysollib.mygettext import _
-from pysollib.pysoltk import TOOLBAR_BUTTONS, TOOLKIT
+from pysollib.mygettext import myGettext
+from pysollib.pysoltk import STATUSBAR_ITEMS, TOOLBAR_BUTTONS, TOOLKIT
 from pysollib.resource import CSI
 
-import six
+import validate
 
 # ************************************************************************
 # * Options
 # ************************************************************************
 
+_global_settings = {
+    'mouse_button1': 1,
+    'mouse_button2': 2,
+    'mouse_button3': 3,
+}
+
+
+def calcCustomMouseButtonsBinding(binding_format):
+    assert _global_settings['mouse_button1']
+    return binding_format.format(
+        mouse_button1=_global_settings['mouse_button1'],
+        mouse_button2=_global_settings['mouse_button2'],
+        mouse_button3=_global_settings['mouse_button3'],
+    )
+
 
 configspec = '''
 [general]
+last_version = list
 player = string
 confirm = boolean
 update_player_stats = boolean
@@ -53,10 +71,13 @@ shuffle = boolean
 undo = boolean
 bookmarks = boolean
 hint = boolean
+free_hint = boolean
 highlight_piles = boolean
 highlight_cards = boolean
 highlight_samerank = boolean
 highlight_not_matching = boolean
+peek_facedown = boolean
+stuck_notification = boolean
 mahjongg_show_removed = boolean
 mahjongg_create_solvable = integer(0, 2)
 shisen_show_hint = boolean
@@ -71,39 +92,50 @@ shade = boolean
 shrink_face_down = boolean
 shade_filled_stacks = boolean
 demo_logo = boolean
+demo_logo_style = string
+pause_text_style = string
+redeal_icon_style = string
+dialog_icon_style = string
+tree_icon_style = string
+button_icon_style = string
 tile_theme = string
 default_tile_theme = string
 toolbar = integer(0, 4)
+toolbar_land = integer(0, 4)
+toolbar_port = integer(0, 4)
 toolbar_style = string
 toolbar_relief = string
 toolbar_compound = string
-toolbar_size = integer(0, 1)
+toolbar_size = integer(0, 2)
 statusbar = boolean
-statusbar_game_number = boolean
-statusbar_stuck = boolean
-num_cards = boolean
-helpbar = boolean
 num_recent_games = integer(10, 100)
 last_gameid = integer
 game_holded = integer
 wm_maximized = boolean
+wm_fullscreen = boolean
 splashscreen = boolean
+date_format = string
 mouse_type = string
 mouse_undo = boolean
 negative_bottom = boolean
 randomize_place = boolean
-save_cardsets = boolean
+use_cardset_bottoms = boolean
 dragcursor = boolean
 save_games_geometry = boolean
 game_geometry = int_list(min=2, max=2)
+topmost_dialogs = boolean
 sound = boolean
 sound_mode = integer(0, 1)
 sound_sample_volume = integer(0, 128)
 sound_sample_buffer_size = integer(1, 4)
+music = boolean
 tabletile_name = string
+tabletile_scale_method = integer
+center_layout = boolean
 recent_gameid = int_list
 favorite_gameid = int_list
 visible_buttons = string_list
+visible_status = string_list
 translate_game_names = boolean
 solver_presets = string_list
 solver_show_progress = boolean
@@ -111,6 +143,8 @@ solver_max_iterations = integer
 solver_iterations_output_step = integer
 solver_preset = string
 display_win_message = boolean
+language = string
+table_zoom = list
 
 [sound_samples]
 move = boolean
@@ -133,6 +167,7 @@ gamewon = boolean
 droppair = boolean
 redo = boolean
 dealwaste = boolean
+extra = boolean
 
 [fonts]
 sans = list
@@ -165,6 +200,7 @@ highlight_piles = float(0.2, 9.9)
 [cardsets]
 0 = string_list(min=2, max=2)
 1 = string_list(min=2, max=2)
+1_1 = string_list(min=2, max=2)
 2 = string_list(min=2, max=2)
 3 = string_list(min=2, max=2)
 4 = string_list(min=2, max=2)
@@ -173,16 +209,29 @@ highlight_piles = float(0.2, 9.9)
 7 = string_list(min=2, max=2)
 8 = string_list(min=2, max=2)
 9 = string_list(min=2, max=2)
+10 = string_list(min=2, max=2)
+11_3 = string_list(min=2, max=2)
+11_4 = string_list(min=2, max=2)
+11_5 = string_list(min=2, max=2)
+11_6 = string_list(min=2, max=2)
+11_7 = string_list(min=2, max=2)
+11_8 = string_list(min=2, max=2)
+11_9 = string_list(min=2, max=2)
+11_10 = string_list(min=2, max=2)
+12 = string_list(min=2, max=2)
 scale_cards = boolean
 scale_x = float
 scale_y = float
 auto_scale = boolean
+spread_stacks = boolean
 preserve_aspect_ratio = boolean
+resampling = integer(0, 10)
 '''.splitlines()
 
 
 class Options:
     GENERAL_OPTIONS = [
+        ('last_version', 'list'),
         ('player', 'str'),
         ('confirm', 'bool'),
         ('update_player_stats', 'bool'),
@@ -194,14 +243,19 @@ class Options:
         ('undo', 'bool'),
         ('bookmarks', 'bool'),
         ('hint', 'bool'),
+        ('free_hint', 'bool'),
         ('highlight_piles', 'bool'),
         ('highlight_cards', 'bool'),
         ('highlight_samerank', 'bool'),
         ('highlight_not_matching', 'bool'),
+        ('peek_facedown', 'bool'),
+        ('stuck_notification', 'bool'),
         ('mahjongg_show_removed', 'bool'),
         ('mahjongg_create_solvable', 'int'),
         ('shisen_show_hint', 'bool'),
         ('shisen_show_matching', 'bool'),
+        ('accordion_deal_all', 'bool'),
+        ('pegged_auto_remove', 'bool'),
         ('animations', 'int'),
         ('redeal_animation', 'bool'),
         ('win_animation', 'bool'),
@@ -212,46 +266,68 @@ class Options:
         ('shrink_face_down', 'bool'),
         ('shade_filled_stacks', 'bool'),
         ('demo_logo', 'bool'),
+        ('demo_logo_style', 'str'),
+        ('pause_text_style', 'str'),
+        ('redeal_icon_style', 'str'),
+        ('dialog_icon_style', 'str'),
+        ('tree_icon_style', 'str'),
+        ('button_icon_style', 'str'),
         ('tile_theme', 'str'),
         ('default_tile_theme', 'str'),
         ('toolbar', 'int'),
+        ('toolbar_land', 'int'),
+        ('toolbar_port', 'int'),
         ('toolbar_style', 'str'),
         ('toolbar_relief', 'str'),
         ('toolbar_compound', 'str'),
         ('toolbar_size', 'int'),
         ('statusbar', 'bool'),
-        ('statusbar_game_number', 'bool'),
-        ('statusbar_stuck', 'bool'),
-        ('num_cards', 'bool'),
-        ('helpbar', 'bool'),
+        # ('statusbar_game_number', 'bool'),
+        # ('statusbar_stuck', 'bool'),
+        # ('num_cards', 'bool'),
+        # ('helpbar', 'bool'),
         ('num_recent_games', 'int'),
         ('last_gameid', 'int'),
         ('game_holded', 'int'),
         ('wm_maximized', 'bool'),
+        ('wm_fullscreen', 'bool'),
         ('splashscreen', 'bool'),
+        ('date_format', 'str'),
         ('mouse_type', 'str'),
         ('mouse_undo', 'bool'),
         ('negative_bottom', 'bool'),
         ('randomize_place', 'bool'),
-        ('save_cardsets', 'bool'),
+        ('use_cardset_bottoms', 'bool'),
+        # ('save_cardsets', 'bool'),
         ('dragcursor', 'bool'),
         ('save_games_geometry', 'bool'),
+        ('topmost_dialogs', 'bool'),
         ('sound', 'bool'),
         ('sound_mode', 'int'),
         ('sound_sample_volume', 'int'),
         ('sound_music_volume', 'int'),
         ('sound_sample_buffer_size', 'int'),
+        ('music', 'bool'),
         ('tabletile_name', 'str'),
+        ('tabletile_scale_method', 'int'),
+        ('center_layout', 'bool'),
         ('translate_game_names', 'bool'),
         ('solver_presets', 'list'),
         ('solver_show_progress', 'bool'),
         ('solver_max_iterations', 'int'),
         ('solver_iterations_output_step', 'int'),
         ('solver_preset', 'string'),
+        ('mouse_button1', 'int'),
+        ('mouse_button2', 'int'),
+        ('mouse_button3', 'int'),
         # ('toolbar_vars', 'list'),
         # ('recent_gameid', 'list'),
         # ('favorite_gameid', 'list'),
         ('display_win_message', 'bool'),
+        ('language', 'str'),
+        # ('table_zoom', 'list'),
+        ('fontscale', 'str'),
+        # ('fontsizefactor', 'float'),
         ]
 
     def __init__(self):
@@ -260,6 +336,8 @@ class Options:
 
         self.version_tuple = pysollib.settings.VERSION_TUPLE  # XXX
         self.saved = 0                  # XXX
+
+        self.last_version = (2, 20, 1)
         # options menu:
         self.player = _("Unknown")
         self.confirm = True
@@ -272,12 +350,17 @@ class Options:
         self.undo = True
         self.bookmarks = True
         self.hint = True
+        self.free_hint = False
         self.highlight_piles = True
         self.highlight_cards = True
         self.highlight_samerank = True
         self.highlight_not_matching = True
+        self.peek_facedown = False
+        self.stuck_notification = False
         self.mahjongg_show_removed = False
         self.mahjongg_create_solvable = 2  # 0 - none, 1 - easy, 2 - hard
+        self.accordion_deal_all = True
+        self.pegged_auto_remove = True
         if TOOLKIT == 'kivy':
             self.mahjongg_create_solvable = 1  # 0 - none, 1 - easy, 2 - hard
         self.shisen_show_hint = True
@@ -295,36 +378,61 @@ class Options:
         self.shrink_face_down = True
         self.shade_filled_stacks = True
         self.demo_logo = True
+        self.demo_logo_style = 'komika'
+        self.pause_text_style = 'komika'
+        self.redeal_icon_style = 'modern'
+        self.dialog_icon_style = 'remix'
+        self.tree_icon_style = 'remix'
+        self.button_icon_style = 'none'
         self.tile_theme = 'default'
         self.default_tile_theme = 'default'
-        self.toolbar = 1       # 0 == hide, 1,2,3,4 == top, bottom, lef, right
+        self.toolbar = 1       # 0 == hide, 1,2,3,4 == top, bottom, left, right
+        # used with 'kivy' version in addition:
+        self.toolbar_land = 4  # (landscape)
+        self.toolbar_port = 2  # (portrait)
+        # 0 == hide,
+        # 1,2,3,4 == top, bottom, left, right
         # self.toolbar_style = 'default'
-        if TOOLKIT == 'kivy':
-            self.toolbar = 4  # 0 == hide, 1,2,3,4 == top, bottom, lef, right
-        self.toolbar_style = 'bluecurve'
+        self.toolbar_style = 'remix'
         self.toolbar_relief = 'flat'
         self.toolbar_compound = 'none'  # icons only
         self.toolbar_size = 0
         self.toolbar_vars = {}
         for w in TOOLBAR_BUTTONS:
             self.toolbar_vars[w] = True  # show all buttons
+        self.statusbar_vars = {}
+        for w, x in STATUSBAR_ITEMS:
+            self.statusbar_vars[w] = True
         self.statusbar = True
-        self.statusbar_game_number = False  # show game number in statusbar
-        self.statusbar_stuck = False        # show stuck indicator
-        self.num_cards = False
-        self.helpbar = False
+        # self.statusbar_game_number = False  # show game number in statusbar
+        # self.statusbar_stuck = False        # show stuck indicator
+        # self.num_cards = False
+        # self.helpbar = False
         self.splashscreen = True
-        self.mouse_type = 'drag-n-drop'  # or 'sticky-mouse' or 'point-n-click'
+        self.date_format = '%m-%d'
+        self.mouse_button1 = 1
+        self.mouse_button2 = 2
+        self.mouse_button3 = 3
+        # mouse_type:  'drag-n-drop' or 'sticky-mouse' or 'point-n-click'
+        if TOOLKIT == 'kivy':
+            self.mouse_type = 'point-n-click'
+        else:
+            self.mouse_type = 'drag-n-drop'
         self.mouse_undo = False         # use mouse for undo/redo
         self.negative_bottom = True
         self.translate_game_names = True
         self.display_win_message = True
+        self.language = ''
+        self.table_zoom = [1.0, 0.0, 0.0]
+        self.fontscale = 'default'        # (kivy,  platform defaults)
+        # self.fontsizefactor = 1.0
         # sound
         self.sound = True
         self.sound_mode = 1
         self.sound_sample_volume = 75
         self.sound_music_volume = 100
         self.sound_sample_buffer_size = 1  # 1 - 4 (1024 - 4096 bytes)
+        self.music = True
         self.sound_samples = {
             'areyousure': True,
             'autodrop': True,
@@ -335,7 +443,7 @@ class Options:
             'dealwaste': True,
             'droppair': True,
             'drop': True,
-            # 'extra': True,
+            'extra': True,
             'flip': True,
             'move': True,
             'nomove': True,
@@ -392,22 +500,32 @@ class Options:
                                     11004, 14405, 14410, 15411, 22225]
         self.last_gameid = 0            # last game played
         self.game_holded = 0            # gameid or 0
-        self.wm_maximized = 0
+        self.wm_maximized = 1
+        self.wm_fullscreen = 0
         self.save_games_geometry = False
+        self.topmost_dialogs = True
         # saved games geometry (gameid: (width, height))
         self.games_geometry = {}
         self.game_geometry = (0, 0)  # game geometry before exit
         self.offsets = {}           # cards offsets
         #
         self.randomize_place = False
-        self.save_cardsets = True
+        self.use_cardset_bottoms = False
+        # self.save_cardsets = True
         self.dragcursor = True
         #
         self.scale_cards = False
         self.scale_x = 1.0
         self.scale_y = 1.0
-        self.auto_scale = False
+        self.auto_scale = True
+        self.spread_stacks = False
+        self.center_layout = True
         self.preserve_aspect_ratio = True
+        self.tabletile_scale_method = 0
+        self.resampling = 0
+        if USE_PIL:
+            self.resampling = int(get_default_resampling())
+
         # solver
         self.solver_presets = [
             'none',
@@ -463,32 +581,71 @@ class Options:
                           top.winfo_screendepth())
         # bg
         if sd > 8:
-            self.tabletile_name = "Nostalgy.gif"  # basename
+            self.tabletile_name = "Felt Green"  # name
         else:
             self.tabletile_name = None
         # cardsets
         c = "Standard"
+        m = "Crystal Mahjongg"
         if sw < 800 or sh < 600:
             c = "2000"
         if TOOLKIT == 'kivy':
             c = "Standard"
+            m = "Gnome Mahjongg 1"
 
         # if sw > 1024 and sh > 768:
         #    c = 'Dondorf'
-        self.cardset = {
-            # game_type:        (cardset_name, back_file)
-            0:                  (c, ""),
-            CSI.TYPE_FRENCH:    (c, ""),
-            CSI.TYPE_HANAFUDA:  ("Kintengu", ""),
-            CSI.TYPE_MAHJONGG:  ("Crystal Mahjongg", ""),
-            CSI.TYPE_TAROCK:    ("Vienna 2K", ""),
-            CSI.TYPE_HEXADECK:  ("Hex A Deck", ""),
-            CSI.TYPE_MUGHAL_GANJIFA: ("Mughal Ganjifa", ""),
-            # CSI.TYPE_NAVAGRAHA_GANJIFA: ("Navagraha Ganjifa", ""),
-            CSI.TYPE_NAVAGRAHA_GANJIFA: ("Dashavatara Ganjifa", ""),
-            CSI.TYPE_DASHAVATARA_GANJIFA: ("Dashavatara Ganjifa", ""),
-            CSI.TYPE_TRUMP_ONLY: ("Matrix", ""),
-        }
+        if USE_PIL:
+            self.cardset = {
+                0:                  {0: ("Neo", "")},
+                CSI.TYPE_FRENCH:    {0: ("Neo", ""), 1: ("Neo", "")},
+                CSI.TYPE_HANAFUDA:  {0: ("Louie Mantia Hanafuda", "")},
+                CSI.TYPE_MAHJONGG:  {0: ("Uni Mahjongg", "")},
+                CSI.TYPE_TAROCK:    {0: ("Neo Tarock", "")},
+                CSI.TYPE_HEXADECK:  {0: ("Neo Hex", "")},
+                CSI.TYPE_MUGHAL_GANJIFA: {0: ("Mughal Ganjifa XL", "")},
+                # CSI.TYPE_NAVAGRAHA_GANJIFA: {0: ("Navagraha Ganjifa", "")},
+                CSI.TYPE_NAVAGRAHA_GANJIFA:
+                    {0: ("Dashavatara Ganjifa XL", "")},
+                CSI.TYPE_DASHAVATARA_GANJIFA:
+                    {0: ("Dashavatara Ganjifa XL", "")},
+                CSI.TYPE_TRUMP_ONLY: {0: ("Next Matrix", "")},
+                CSI.TYPE_MATCHING: {0: ("Neo", "")},
+                CSI.TYPE_PUZZLE: {3: ("Dojouji Ukiyo E (3x3)", ""),
+                                  4: ("Knave of Hearts (4x4)", ""),
+                                  5: ("Victoria Falls (5x5)", ""),
+                                  6: ("Hokusai Ukiyo E (6x6)", ""),
+                                  7: ("Blaren (7x7)", ""),
+                                  8: ("Mid Winter's Eve (8x8)", ""),
+                                  9: ("The Card Players (9x9)", ""),
+                                  10: ("Players Trumps (10x10)", "")},
+                CSI.TYPE_ISHIDO: {0: ("Simple Ishido XL", "")},
+            }
+        else:
+            self.cardset = {
+                # game_type:        (cardset_name, back_file)
+                0:                  {0: (c, "")},
+                CSI.TYPE_FRENCH:    {0: (c, ""), 1: (c, "")},
+                CSI.TYPE_HANAFUDA:  {0: ("Kintengu", "")},
+                CSI.TYPE_MAHJONGG:  {0: (m, "")},
+                CSI.TYPE_TAROCK:    {0: ("Vienna 2K", "")},
+                CSI.TYPE_HEXADECK:  {0: ("Hex A Deck", "")},
+                CSI.TYPE_MUGHAL_GANJIFA: {0: ("Mughal Ganjifa", "")},
+                # CSI.TYPE_NAVAGRAHA_GANJIFA: {0: ("Navagraha Ganjifa", "")},
+                CSI.TYPE_NAVAGRAHA_GANJIFA: {0: ("Dashavatara Ganjifa", "")},
+                CSI.TYPE_DASHAVATARA_GANJIFA: {0: ("Dashavatara Ganjifa", "")},
+                CSI.TYPE_TRUMP_ONLY: {0: ("Matrix", "")},
+                CSI.TYPE_MATCHING: {0: (c, "")},
+                CSI.TYPE_PUZZLE: {3: ("Dojouji Ukiyo E (3x3)", ""),
+                                  4: ("Knave of Hearts (4x4)", ""),
+                                  5: ("Victoria Falls (5x5)", ""),
+                                  6: ("Hokusai Ukiyo E (6x6)", ""),
+                                  7: ("Blaren (7x7)", ""),
+                                  8: ("Mid Winter's Eve (8x8)", ""),
+                                  9: ("The Card Players (9x9)", ""),
+                                  10: ("Players Trumps (10x10)", "")},
+                CSI.TYPE_ISHIDO: {0: ("Simple Ishido", "")},
+            }
 
     # not changeable options
     def setConstants(self):
@@ -509,19 +666,25 @@ class Options:
     def save(self, filename):
         config = self._config
 
+        self.last_version = self.version_tuple
+
         # general
         for key, t in self.GENERAL_OPTIONS:
             val = getattr(self, key)
             if isinstance(val, str):
                 if sys.version_info < (3,):
-                    val = six.text_type(val, 'utf-8')
+                    val = str(val, 'utf-8')
             config['general'][key] = val
 
         config['general']['recent_gameid'] = self.recent_gameid
         config['general']['favorite_gameid'] = self.favorite_gameid
+        config['general']['table_zoom'] = self.table_zoom
         visible_buttons = [b for b in self.toolbar_vars
                            if self.toolbar_vars[b]]
         config['general']['visible_buttons'] = visible_buttons
+        visible_status = [b for b in self.statusbar_vars
+                          if self.statusbar_vars[b]]
+        config['general']['visible_status'] = visible_status
         if 'none' in config['general']['solver_presets']:
             config['general']['solver_presets'].remove('none')
 
@@ -544,9 +707,14 @@ class Options:
 
         # cardsets
         for key, val in self.cardset.items():
-            config['cardsets'][str(key)] = val
+            for key2, val2 in val.items():
+                if key2 > 0:
+                    config['cardsets'][str(key) + "_" + str(key2)] = val2
+                else:
+                    config['cardsets'][str(key)] = val2
         for key in ('scale_cards', 'scale_x', 'scale_y',
-                    'auto_scale', 'preserve_aspect_ratio'):
+                    'auto_scale', 'spread_stacks',
+                    'preserve_aspect_ratio', 'resampling'):
             config['cardsets'][key] = getattr(self, key)
 
         # games_geometry
@@ -624,14 +792,12 @@ class Options:
         vdt = validate.Validator()
         res = config.validate(vdt)
         # from pprint import pprint; pprint(res)
-        if res is not True:
+        if isinstance(res, dict):
             for section, data in res.items():
                 if data is True:
                     continue
                 for key, value in data.items():
                     if value is False:
-                        print_err('config file: validation error: '
-                                  'section: "%s", key: "%s"' % (section, key))
                         config[section][key] = None
 
         # general
@@ -658,10 +824,23 @@ class Options:
             except Exception:
                 traceback.print_exc()
 
+        table_zoom = self._getOption('general', 'table_zoom', 'list')
+        if table_zoom is not None:
+            try:
+                self.table_zoom = [float(i) for i in table_zoom]
+            except Exception:
+                traceback.print_exc()
+
         visible_buttons = self._getOption('general', 'visible_buttons', 'list')
         if visible_buttons is not None:
             for key in TOOLBAR_BUTTONS:
                 self.toolbar_vars[key] = (key in visible_buttons)
+        visible_status = self._getOption('general', 'visible_status', 'list')
+        if visible_status is not None:
+            for key, label in STATUSBAR_ITEMS:
+                self.statusbar_vars[key] = (key in visible_status)
+
+        myGettext.language = self.language
 
         # solver
         solver_presets = self._getOption('general', 'solver_presets', 'list')
@@ -704,17 +883,24 @@ class Options:
 
         # cardsets
         for key in self.cardset:
-            val = self._getOption('cardsets', str(key), 'list')
-            if val is not None:
-                try:
-                    self.cardset[int(key)] = val
-                except Exception:
-                    traceback.print_exc()
+            for key2 in self.cardset[key]:
+                if key2 > 0:
+                    val = self._getOption('cardsets',
+                                          str(key) + "_" + str(key2), 'list')
+                else:
+                    val = self._getOption('cardsets', str(key), 'list')
+                if val is not None:
+                    try:
+                        self.cardset[int(key)][int(key2)] = val
+                    except Exception:
+                        traceback.print_exc()
         for key, t in (('scale_cards', 'bool'),
                        ('scale_x', 'float'),
                        ('scale_y', 'float'),
                        ('auto_scale', 'bool'),
-                       ('preserve_aspect_ratio', 'bool')):
+                       ('spread_stacks', 'bool'),
+                       ('preserve_aspect_ratio', 'bool'),
+                       ('resampling', 'int')):
             val = self._getOption('cardsets', key, t)
             if val is not None:
                 setattr(self, key, val)
@@ -742,3 +928,20 @@ class Options:
                 self.offsets[key] = val
             except Exception:
                 traceback.print_exc()
+
+        # mouse buttons swap
+        def _positive(button):
+            return max([button, 1])
+        _global_settings['mouse_button1'] = _positive(self.mouse_button1)
+        _global_settings['mouse_button2'] = _positive(self.mouse_button2)
+        _global_settings['mouse_button3'] = _positive(self.mouse_button3)
+
+    def calcCustomMouseButtonsBinding(self, binding_format):
+        """docstring for calcCustomMouseButtonsBinding"""
+        def _positive(button):
+            return max([button, 1])
+        return binding_format.format(
+            mouse_button1=_positive(self.mouse_button1),
+            mouse_button2=_positive(self.mouse_button2),
+            mouse_button3=_positive(self.mouse_button3),
+        )
