@@ -25,12 +25,10 @@ import glob
 import os
 import traceback
 
-from pysollib.mfxutil import KwStruct, Struct
+from pysollib.mfxutil import Image, KwStruct, Struct, USE_PIL
 from pysollib.mygettext import _
 from pysollib.settings import DEBUG
 
-import six
-
 # ************************************************************************
 # * Abstract
 # ************************************************************************
@@ -185,8 +183,10 @@ class CSI:
     TYPE_ISHIDO = 12
 
     # cardset subtypes
+    # (french)
     SUBTYPE_NONE = 0
     SUBTYPE_JOKER_DECK = 1
+    # (puzzle)
     SUBTYPE_3X3 = 3
     SUBTYPE_4X4 = 4
     SUBTYPE_5X5 = 5
@@ -475,7 +475,7 @@ class Cardset(Resource):
 
     def updateCardback(self, backname=None, backindex=None):
         # update default back
-        if isinstance(backname, six.string_types):
+        if isinstance(backname, str):
             if backname in self.backnames:
                 backindex = self.backnames.index(backname)
         if isinstance(backindex, int):
@@ -574,7 +574,7 @@ class CardsetManager(ResourceManager):
             return
         cs.ncards = len(cs.ranks) * len(cs.suits) + len(cs.trumps)
         cs.name = cs.name[:30]
-        if not (1 <= cs.si.size <= 5):
+        if not (CSI.SIZE_TINY <= cs.si.size <= CSI.SIZE_HIRES):
             CW, CH = cs.CARDW, cs.CARDH
             if CW <= 55 and CH <= 72:
                 cs.si.size = CSI.SIZE_TINY
@@ -675,16 +675,45 @@ class CardsetManager(ResourceManager):
 # * Tile
 # ************************************************************************
 
+# TableTileInfo constants
+class TTI:
+    # tile size
+    SIZE_UNKNOWN = 0
+    SIZE_TILE = 1
+    SIZE_SD = 2
+    SIZE_HD = 3
+    SIZE_4K = 4
+
+
 class Tile(Resource):
     def __init__(self, **kw):
         kw['color'] = None
         kw['stretch'] = 0
         kw['save_aspect'] = 0
+        kw['size'] = 0
         Resource.__init__(self, **kw)
 
 
 class TileManager(ResourceManager):
-    pass
+    def register(self, tile):
+        if USE_PIL:
+            try:
+                img = Image.open(tile.filename)
+                TW, TH = img.size
+                if TW < 640 or TH < 480:
+                    tile.size = TTI.SIZE_TILE
+                elif TW < 1280 or TH < 720:
+                    tile.size = TTI.SIZE_SD
+                elif TW < 3840 or TH < 2160:
+                    tile.size = TTI.SIZE_HD
+                else:
+                    tile.size = TTI.SIZE_4K
+            except AttributeError:
+                tile.size = TTI.SIZE_UNKNOWN
+        else:
+            tile.size = TTI.SIZE_UNKNOWN
+
+        ResourceManager.register(self, tile)
 
 
 # ************************************************************************
