@@ -24,6 +24,9 @@
 import locale
 import os
 import time
+import tkinter
+import tkinter.font
+import tkinter.ttk as ttk
 import traceback
 
 from pysollib.mfxutil import KwStruct, destruct, kwdefault, openURL
@@ -34,13 +37,6 @@ from pysollib.ui.tktile.tkutil import after, after_cancel
 from pysollib.ui.tktile.tkutil import bind, unbind_destroy
 from pysollib.ui.tktile.tkutil import makeToplevel, setTransient
 
-import six
-from six import PY2
-from six.moves import tkinter
-from six.moves import tkinter_font
-
-from . import ttk
-
 # ************************************************************************
 # * abstract base class for the dialogs in this module
 # ************************************************************************
@@ -66,7 +62,7 @@ class MfxDialog:  # ex. _ToplevelDialog
         # self.top.wm_maxsize(w-4, h-32)
         bind(self.top, "WM_DELETE_WINDOW", self.wmDeleteWindow)
 
-    def mainloop(self, focus=None, timeout=0, transient=True):
+    def mainloop(self, focus=None, timeout=0, transient=True, geometry=""):
         bind(self.top, "<Escape>", self.mCancel)
         bind(self.top, '<Alt-Key>', self.altKeyEvent)  # for accelerators
         if focus is not None:
@@ -79,6 +75,8 @@ class MfxDialog:  # ex. _ToplevelDialog
                 if traceback:
                     traceback.print_exc()
                 pass
+            if geometry != "":
+                self.top.geometry(geometry)
             if timeout > 0:
                 self.timer = after(self.top, timeout, self.mTimeout)
             try:
@@ -120,9 +118,9 @@ class MfxDialog:  # ex. _ToplevelDialog
             key = event.char
             try:
                 if os.name == 'nt':
-                    key = six.text_type(key, locale.getpreferredencoding())
+                    key = str(key, locale.getpreferredencoding())
                 else:
-                    key = six.text_type(key, 'utf-8')
+                    key = str(key, 'utf-8')
             except Exception:
                 pass
             else:
@@ -156,7 +154,7 @@ class MfxDialog:  # ex. _ToplevelDialog
             separator = ttk.Separator(self._frame)
             separator.pack(side='bottom', fill='x')
         top_frame = ttk.Frame(self._frame)
-        top_frame.pack(side='top', fill='both', expand=1)
+        top_frame.pack(side='top', fill='both', expand=True)
         return top_frame, bottom_frame
 
     def createBitmaps(self, frame, kw):
@@ -288,8 +286,6 @@ class MfxExceptionDialog(MfxMessageDialog):
                 (ex.errno, ex.strerror, repr(ex.filename))
         else:
             t = str(ex)
-        if PY2:
-            t = six.text_type(t, errors='replace')
         kw.text = text + t
         MfxMessageDialog.__init__(self, parent, title, **kw.getKw())
 
@@ -301,6 +297,7 @@ class MfxExceptionDialog(MfxMessageDialog):
 class PysolAboutDialog(MfxMessageDialog):
     def __init__(self, app, parent, title, **kw):
         self._url = kw['url']
+        self.app = app
         kw = self.initKw(kw)
         MfxDialog.__init__(self, parent, title, kw.resizable, kw.default)
         top_frame, bottom_frame = self.createFrames(kw)
@@ -315,20 +312,38 @@ class PysolAboutDialog(MfxMessageDialog):
 
         # font_name = msg.lookup('TLabel', 'font')
         font_name = 'TkDefaultFont'
-        font = tkinter_font.Font(parent, name=font_name, exists=True)
+        font = tkinter.font.Font(parent, name=font_name, exists=True)
         font = font.copy()
         font.configure(underline=True)
         url_label = ttk.Label(frame, text=kw.url, font=font,
                               foreground='blue', cursor='hand2')
         url_label.pack()
-        url_label.bind('<1>', self._urlClicked)
+        from pysollib.options import calcCustomMouseButtonsBinding
+        url_label.bind(
+            calcCustomMouseButtonsBinding('<{mouse_button1}>'),
+            self._urlClicked
+        )
         #
+
         focus = self.createButtons(bottom_frame, kw)
+
+        self.splashscreen = tkinter.BooleanVar()
+        self.splashscreen.set(app.opt.splashscreen)
+        show_on_start = ttk.Checkbutton(bottom_frame,
+                                        variable=self.splashscreen,
+                                        command=self._splashUpdate,
+                                        text=_("Show this on startup"))
+        show_on_start.grid(row=0, column=0, sticky='w',
+                           padx=1, pady=1)
+
         self.mainloop(focus, kw.timeout)
 
     def _urlClicked(self, event):
         openURL(self._url)
 
+    def _splashUpdate(self):
+        self.app.opt.splashscreen = self.splashscreen.get()
+
 
 # ************************************************************************
 # *
@@ -505,7 +520,7 @@ class MfxScrolledCanvas:
     #
     #
 
-    def setTile(self, app, i, force=False):
+    def setTile(self, app, i, scale_method, force=False):
         tile = app.tabletile_manager.get(i)
         if tile is None or tile.error:
             return False
@@ -522,8 +537,19 @@ class MfxScrolledCanvas:
                     tile.color == app.opt.colors['table']):
                 return False
         #
-        if not self.canvas.setTile(tile.filename, tile.stretch,
-                                   tile.save_aspect):
+        stretch = tile.stretch
+        save_aspect = tile.save_aspect
+
+        if scale_method < 0 and stretch:
+            scale_method = app.opt.tabletile_scale_method
+        elif not stretch:
+            scale_method = 0
+
+        if scale_method > 0:
+            stretch = scale_method > 1
+            save_aspect = scale_method > 2
+
+        if not self.canvas.setTile(tile.filename, stretch, save_aspect):
             tile.error = True
             return False
 
@@ -609,7 +635,9 @@ class MfxScrolledCanvas:
         if self.canvas.busy:
             return
         sb = self.hbar
-        if float(first) <= 0 and float(last) >= 1:
+        # TODO - Setting this to .99 takes the scrollbar size into account.
+        # But there is probably a better way to do it.
+        if float(first) <= 0 and float(last) >= .99:
             sb.grid_remove()
             self.hbar_show = False
         else:
@@ -622,7 +650,8 @@ class MfxScrolledCanvas:
         if self.canvas.busy:
             return
         sb = self.vbar
-        if float(first) <= 0 and float(last) >= 1:
+        # TODO - See _setHbar above.
+        if float(first) <= 0 and float(last) >= .99:
             sb.grid_remove()
             self.vbar_show = False
         else:
