@@ -21,18 +21,15 @@
 #
 # ---------------------------------------------------------------------------##
 
-# imports
 import os
-from six.moves import tkinter
+import tkinter
 
-# PySol imports
-from pysollib.mygettext import _
 from pysollib.mfxutil import KwStruct
-from pysollib.settings import TITLE
+from pysollib.mygettext import _
 from pysollib.pysolaudio import pysolsoundserver
-
-# Toolkit imports
+from pysollib.settings import TITLE
 from pysollib.ui.tktile.tkconst import EVENT_HANDLED
+
 from .tkwidget import MfxDialog, MfxMessageDialog
 
 # ************************************************************************
@@ -86,6 +83,8 @@ class SoundOptionsDialog(MfxDialog):
             ('gamelost',      _('Game lost'),      tkinter.BooleanVar()),
             ('gamewon',       _('Game won'),       tkinter.BooleanVar()),
             ('gameperfect',   _('Perfect game'),   tkinter.BooleanVar()),
+
+            ('extra',         _('Other'),          tkinter.BooleanVar()),
             ]
 
         #
